use std::collections::BTreeSet;
use std::io;
use std::iter::{repeat, successors};

fn next(c: (i32, i32), direction: &str) -> (i32, i32) {
    match direction {
        "R" => (c.0 + 1, c.1),
        "L" => (c.0 - 1, c.1),
        "U" => (c.0, c.1 + 1),
        "D" => (c.0, c.1 - 1),
        _ => panic!("Wrong direction format!"),
    }
}

fn direction_length(entry: String) -> (String, usize) {
    (
        entry.chars().nth(0).unwrap().to_string(),
        entry[1..].parse().unwrap(),
    )
}

fn to_coordinates(entries: Vec<String>) -> BTreeSet<(i32, i32)> {
    let mut wire = BTreeSet::new();

    let mut it = entries
        .into_iter()
        .map(direction_length)
        .flat_map(|(direction, length)| repeat(direction).take(length));

    successors(
        it.next()
            .map(|direction| (next((0, 0), direction.as_ref()))),
        |c| it.next().map(|direction| (next(*c, direction.as_ref()))),
    )
    .for_each(|c| {
        wire.insert(c);
    });

    wire
}

fn distance(position: (i32, i32)) -> i32 {
    position.0.abs() + position.1.abs()
}

fn closest(input: String) -> Option<i32> {
    let entries: Vec<BTreeSet<(i32, i32)>> = input
        .trim()
        .split("\n")
        .map(|line| line.split(",").map(String::from).collect())
        .map(to_coordinates)
        .collect();

    entries[0]
        .intersection(&entries[1])
        .cloned()
        .map(distance)
        .min()
}

fn first(input: String) -> Option<i32> {}

pub fn main() -> io::Result<()> {
    let closest = closest(std::include_str!("../day3-input.txt").to_owned());
    println!("{:?}", closest);

    Ok(())
}

#[test]
fn test_coordinates_for_entry() {
    let mut expected: BTreeSet<(i32, i32)> = BTreeSet::new();
    for i in 1..11 {
        expected.insert((i, 0));
    }
    assert_eq!(to_coordinates(vec!["R10".to_owned()]), expected);

    expected = BTreeSet::new();
    expected.insert((1, 0));
    expected.insert((2, 0));
    expected.insert((2, 1));
    expected.insert((2, 2));
    let input = "R2,U2".to_owned().split(",").map(String::from).collect();
    assert_eq!(to_coordinates(input), expected);
}

#[test]
fn test_part_one() {
    let input = "R75,D30,R83,U83,L12,D49,R71,U7,L72\n\
                 U62,R66,U55,R34,D71,R55,D58,R83"
        .to_owned();
    assert_eq!(closest(input), Some(159));
    let input = "R98,U47,R26,D63,R33,U87,L62,D20,R33,U53,R51\n\
                 U98,R91,D20,R16,D67,R40,U7,R15,U6,R7"
        .to_owned();
    assert_eq!(closest(input), Some(135));
}
